@@ -613,6 +613,11 @@ class CommitServicer(CommitServiceServicer, HGitalyServicer):
                 context, StatusCode.NOT_FOUND, "commits not found",
                 FindCommitsError())
 
+        message_regex = request.message_regex
+        if message_regex:
+            grep = f" and grep('(?i){message_regex}')"
+            opts[b'rev'][0] += grep.encode('utf-8')
+
         walk_opts = logcmdutil.parseopts(repo.ui, pats, opts)
         revs, _ = logcmdutil.getrevs(repo, walk_opts)
 
