@@ -525,7 +525,8 @@ class DiffServicer(DiffServiceServicer, HGitalyServicer):
         for paths in chunked(path for extr in extracted
                              for path in changed_paths(
                                      repo, *extr,
-                                     find_renames=find_renames)):
+                                     find_renames=find_renames,
+                                     diff_filters=request.diff_filters)):
             yield FindChangedPathsResponse(paths=paths)
 
     def GetPatchID(self, request: GetPatchIDRequest,
