@@ -361,9 +361,14 @@ class OperationServicer(OperationServiceServicer, HGitalyServicer):
             start_ctx = repo[ZERO_SHA]
         if commit_header.branch_name:
             old_oid = commit_header.expected_old_oid.encode('ascii')
-            if old_oid and start_ctx.hex() != old_oid:
-                context.abort(StatusCode.INVALID_ARGUMENT,
-                              "wrong old oid")
+            if old_oid:
+                old_ctx = gitlab_revision_changeset(repo, gl_branch)
+                if (
+                        (old_ctx is None and old_oid != ZERO_SHA)
+                        or (old_ctx is not None and old_ctx.hex() != old_oid)
+                ):
+                    context.abort(StatusCode.INVALID_ARGUMENT,
+                                  "wrong old oid")
 
         if to_publish and self.heptapod_permission != 'publish':
             context.abort(
@@ -426,7 +431,9 @@ class OperationServicer(OperationServiceServicer, HGitalyServicer):
 
                     if header.action in (ActionType.CREATE,
                                          ActionType.UPDATE):
-                        content_handler = UserCommitFilesContent(wd, header)
+                        content_handler = UserCommitFilesContent(
+                            context, wd, header
+                        )
                     else:
                         content_handler = None
 
@@ -477,6 +484,31 @@ class OperationServicer(OperationServiceServicer, HGitalyServicer):
         )
 
 
+def validate_checkout_path(context, relpath):
+    # absolute paths are interpreted by Gitaly as relative to the
+    # root of checkout.
+    relpath = relpath.lstrip(b'/')
+
+    for p in FORBIDDEN_IN_PATHS:
+        if p in relpath:
+            relpath_str = relpath.decode('utf-8', 'surrogateescape')
+
+            if p == DIRECTORY_CLIMB_UP:
+                # ill-named by upstream
+                error_type = 'ERROR_TYPE_DIRECTORY_TRAVERSAL'
+                msg = 'Path cannot include directory traversal'
+            else:
+                error_type = 'ERROR_TYPE_INVALID_PATH'
+                msg = f'invalid path: "{relpath_str}"'
+
+            index_error(context,
+                        status_code='INVALID_ARGUMENT',
+                        error_type=error_type,
+                        msg=msg,
+                        path=relpath)
+    return relpath
+
+
 class UserCommitFilesAction:
 
     def __init__(self, context, header, working_dir, changed_files):
@@ -485,9 +517,8 @@ class UserCommitFilesAction:
         self.changed_files = changed_files
 
         wd = self.working_dir = working_dir
-        relpath = self.relpath = header.file_path
-        self.validate(relpath)
-        self.abspath = wd.file_path(relpath)
+        self.relpath = validate_checkout_path(context, header.file_path)
+        self.abspath = wd.file_path(self.relpath)
 
     def __call__(self):
         action = self.header.action
@@ -504,36 +535,6 @@ class UserCommitFilesAction:
         elif action == ActionType.CHMOD:
             return self.chmod()
 
-    def validate(self, relpath):
-        if os.path.isabs(relpath):
-            index_error(
-                self.context,
-                status_code='INVALID_ARGUMENT',
-                error_type='ERROR_TYPE_INVALID_PATH',
-                msg='absolute',
-                path=relpath,
-            )
-
-        for p in FORBIDDEN_IN_PATHS:
-            if p in relpath:
-                relpath_str = relpath.decode('utf-8', 'surrogateescape')
-
-                if p == DIRECTORY_CLIMB_UP:
-                    # ill-named by upstream
-                    error_type = 'ERROR_TYPE_DIRECTORY_TRAVERSAL'
-                    msg = 'Path cannot include directory traversal'
-                else:
-                    error_type = 'ERROR_TYPE_INVALID_PATH'
-                    msg = f'invalid path: "{relpath_str}"'
-
-                index_error(
-                    self.context,
-                    status_code='INVALID_ARGUMENT',
-                    error_type=error_type,
-                    msg=msg,
-                    path=relpath,
-                )
-
     def create(self):
         if os.path.exists(self.abspath):
             index_error(
@@ -600,8 +601,8 @@ class UserCommitFilesAction:
     def move(self):
         self.require_file_absence()
         abspath = self.abspath
-        prev_relpath = self.header.previous_path
-        self.validate(prev_relpath)
+        prev_relpath = validate_checkout_path(self.context,
+                                              self.header.previous_path)
         prev_abspath = self.working_dir.file_path(prev_relpath)
         self.require_file_existence(abspath=prev_abspath,
                                     relpath=prev_relpath)
@@ -636,8 +637,9 @@ class UserCommitFilesAction:
 
 class UserCommitFilesContent:
 
-    def __init__(self, workdir, header):
-        self.file_path = workdir.file_path(header.file_path)
+    def __init__(self, context, workdir, header):
+        relpath = validate_checkout_path(context, header.file_path)
+        self.file_path = workdir.file_path(relpath)
         self.fobj = open(self.file_path, 'wb')
         self.make_hg_executable = (header.action == ActionType.CREATE
                                    and header.execute_filemode)
