@@ -368,6 +368,8 @@ def test_find_commits(commit_fixture_empty_repo):
         assert exc_info.value.code() == grpc.StatusCode.NOT_FOUND
     # with all, return all the commits
     assert find_commits_ids(all=True) == [sha0, sha1, sha2, sha3, sha4]
+    # with message_regex
+    assert find_commits_ids(all=True, message_regex='FOO.*zoO') == [sha2]
     # with offset
     assert find_commits_ids(all=True, offset=2) == [sha2, sha3, sha4]
     # with skip_merges
