@@ -34,6 +34,13 @@ from hgitaly.stub.diff_pb2_grpc import DiffServiceStub
 from .fixture import ServiceFixture
 
 StatusCode = grpc.StatusCode
+DiffStatus = FindChangedPathsRequest.DiffStatus
+
+DS_ADDED = DiffStatus.DIFF_STATUS_ADDED
+DS_MODIFIED = DiffStatus.DIFF_STATUS_MODIFIED
+DS_DELETED = DiffStatus.DIFF_STATUS_DELETED
+DS_COPIED = DiffStatus.DIFF_STATUS_COPIED
+DS_RENAMED = DiffStatus.DIFF_STATUS_RENAMED
 
 
 class DiffFixture(ServiceFixture):
@@ -170,9 +177,9 @@ def test_raw_diff(diff_fixture):
                                message=b'added foo')
     ctx1 = wrapper.commit_file('foo', content="I am foo\n",
                                message=b'changes foo')
-    wrapper.command(b'mv', wrapper.repo.root + b'/foo',
+    wrapper.command('mv', wrapper.repo.root + b'/foo',
                     wrapper.repo.root + b'/zoo')
-    wrapper.command(b'ci', message=b"rename foo to zoo")
+    wrapper.command('ci', message=b"rename foo to zoo")
     ctx2 = wrapper.repo[b'.']
     sha0, sha1, sha2 = ctx0.hex(), ctx1.hex(), ctx2.hex()
 
@@ -367,7 +374,7 @@ def test_commit_diff(diff_fixture):
                                message="Add bar")
     ctx1 = wrapper.commit_file('bar', content="I am in\nbar\n",
                                message="Changes bar")
-    wrapper.command(b'mv', wrapper.repo.root + b'/bar',
+    wrapper.command('mv', wrapper.repo.root + b'/bar',
                     wrapper.repo.root + b'/zar')
     ctx2 = wrapper.commit([b'bar', b'zar'], message="Rename bar to zar")
     ctx3 = wrapper.commit_file('zoo', content="I am in\nzoo\n",
@@ -470,7 +477,7 @@ def test_commit_delta(diff_fixture):
                                message="Add bar")
     ctx1 = wrapper.commit_file('bar', content="I am in\nbar\n",
                                message="Changes bar")
-    wrapper.command(b'mv', wrapper.repo.root + b'/bar',
+    wrapper.command('mv', wrapper.repo.root + b'/bar',
                     wrapper.repo.root + b'/zar')
     ctx2 = wrapper.commit([b'bar', b'zar'], message="Rename bar to zar")
     ctx3 = wrapper.commit_file('zoo', content="I am in\nzoo\n",
@@ -545,7 +552,7 @@ def test_diff_stats(diff_fixture):
                         message="Add bar")
     ctx1 = wrapper.commit_file('bar', content="I am in\nbar\n",
                                message="Changes bar")
-    wrapper.command(b'mv', wrapper.repo.root + b'/bar',
+    wrapper.command('mv', wrapper.repo.root + b'/bar',
                     wrapper.repo.root + b'/zar')
     ctx2 = wrapper.commit([b'bar', b'zar'], message="Rename bar to zar")
     ctx3 = wrapper.commit_file('zoo', content="I am in\nzoo\n",
@@ -586,8 +593,15 @@ def test_diff_stats(diff_fixture):
     assert exc_info.value.code() == grpc.StatusCode.FAILED_PRECONDITION
 
 
+def sub_dict(d, *keys):
+    """Sub dict with given keys, after encoding to bytes."""
+    return {k.encode('ascii'): d[k.encode('ascii')]
+            for k in keys}
+
+
 def test_find_changed_paths(diff_fixture):
     wrapper = diff_fixture.repo_wrapper
+    repo = wrapper.repo
 
     (wrapper.path / 'sub').mkdir()
     (wrapper.path / 'sub/foo').write_text('foo content')
@@ -599,8 +613,7 @@ def test_find_changed_paths(diff_fixture):
     (wrapper.path / 'too').chmod(0o755)
     (wrapper.path / 'sub/foo').write_text('foo content modified')
     (wrapper.path / 'bar').unlink()
-    wrapper.command(b'cp', wrapper.repo.root + b'/zoo',
-                    wrapper.repo.root + b'/zaz')
+    wrapper.command('cp', repo.root + b'/zoo', repo.root + b'/zaz')
     ctx1 = wrapper.commit(rel_paths=['sub/foo', 'bar', 'zaz', 'too'],
                           add_remove=True)
 
@@ -635,6 +648,15 @@ def test_find_changed_paths(diff_fixture):
     assert diff_fixture.find_changed_paths_commits(
         [ctx1.hex(), ctx2.hex()]) == resp_dict
 
+    # diff_filters
+    def fcp_filtered(filters):
+        return diff_fixture.find_changed_paths_commits(
+            [ctx1.hex(), ctx2.hex()], diff_filters=filters)
+
+    assert fcp_filtered([DS_ADDED]) == sub_dict(resp_dict, 'too')
+    assert fcp_filtered([DS_MODIFIED]) == sub_dict(resp_dict, 'sub/foo')
+    assert fcp_filtered([DS_COPIED]) == sub_dict(resp_dict, 'zaz')
+
     # Testing by passing "parents" (changesets to compare to)
     ctx3 = wrapper.commit_file("toto")
     assert diff_fixture.find_changed_paths_commits(
@@ -648,7 +670,6 @@ def test_find_changed_paths(diff_fixture):
         }
 
     # Tree requests
-    repo = wrapper.repo
     sub0_oid, sub1_oid = [tree_oid(repo, ctx.hex().decode('ascii'), b'sub')
                           for ctx in (ctx0, ctx1)]
     assert diff_fixture.find_changed_paths_tree(sub0_oid, sub1_oid) == {
@@ -669,17 +690,16 @@ def test_find_changed_paths(diff_fixture):
 
 def test_find_changed_paths_copy_in_tree(diff_fixture):
     wrapper = diff_fixture.repo_wrapper
+    repo = wrapper.repo
 
     (wrapper.path / 'subdir').mkdir()  # avoid all lengths to be 3
     (wrapper.path / 'subdir/bar').write_text('some bar')
     ctx0 = wrapper.commit(rel_paths=['subdir'], add_remove=True)
 
-    wrapper.command(b'cp', wrapper.repo.root + b'/subdir/bar',
-                    wrapper.repo.root + b'/subdir/baz')
+    wrapper.command('cp', repo.root + b'/subdir/bar',
+                    repo.root + b'/subdir/baz')
     ctx1 = wrapper.commit(rel_paths=['subdir'])
 
-    repo = wrapper.repo
-
     sub0_oid, sub1_oid = [tree_oid(repo, ctx.hex().decode('ascii'), b'subdir')
                           for ctx in (ctx0, ctx1)]
     assert diff_fixture.find_changed_paths_tree(sub0_oid, sub1_oid) == {
@@ -689,25 +709,23 @@ def test_find_changed_paths_copy_in_tree(diff_fixture):
 
 def test_find_changed_paths_rename(diff_fixture):
     wrapper = diff_fixture.repo_wrapper
+    repo = wrapper.repo
 
     (wrapper.path / 'subdir').mkdir()  # avoid all lengths to be 3
     (wrapper.path / 'subdir/bar').write_text('some bar')
     ctx0 = wrapper.commit(rel_paths=['subdir'], add_remove=True)
 
-    wrapper.command(b'mv', wrapper.repo.root + b'/subdir/bar',
-                    wrapper.repo.root + b'/subdir/baz')
+    wrapper.command('mv', repo.root + b'/subdir/bar',
+                    repo.root + b'/subdir/baz')
     ctx1 = wrapper.commit(rel_paths=['subdir'])
 
-    repo = wrapper.repo
+    def fcp(**opts):
+        return diff_fixture.find_changed_paths_commits(
+            [ctx1.hex()],
+            compare_to=[ctx0.hex()],
+            **opts)
 
-    sub0_oid, sub1_oid = [tree_oid(repo, ctx.hex().decode('ascii'), b'subdir')
-                          for ctx in (ctx0, ctx1)]
-    ret = diff_fixture.find_changed_paths_commits(
-        [ctx1.hex()],
-        compare_to=[ctx0.hex()],
-        find_renames=False,
-    )
-    assert ret == {
+    expected_no_renames = {
         b'subdir/baz': [(ChangedPaths.Status.ADDED,
                          (OBJECT_MODE_DOES_NOT_EXIST,
                           OBJECT_MODE_NON_EXECUTABLE),
@@ -718,17 +736,50 @@ def test_find_changed_paths_rename(diff_fixture):
                          )],
     }
 
-    ret = diff_fixture.find_changed_paths_commits(
-        [ctx1.hex()],
-        compare_to=[ctx0.hex()],
-        find_renames=True,
-    )
-    assert ret == {
+    assert fcp(find_renames=False) == expected_no_renames
+    assert fcp(find_renames=False,
+               diff_filters=[DS_ADDED],
+               ) == sub_dict(expected_no_renames, 'subdir/baz')
+    assert fcp(find_renames=False,
+               diff_filters=[DS_DELETED],
+               ) == sub_dict(expected_no_renames, 'subdir/bar')
+
+    expected_renames = {
         b'subdir/baz': [(ChangedPaths.Status.RENAMED, None)],
     }
+    assert fcp(find_renames=True) == expected_renames
+    assert fcp(find_renames=True,
+               diff_filters=[DS_ADDED, DS_MODIFIED, DS_DELETED]
+               ) == {}
+    assert fcp(find_renames=True,
+               diff_filters=[DS_RENAMED]) == expected_renames
 
-    # TODO test the case of rename with duplication:
-    # hg cp foo bar; hg mv foo bar2
+
+def test_find_changed_paths_rename_and_copy(diff_fixture):
+    wrapper = diff_fixture.repo_wrapper
+    repo = wrapper.repo
+
+    ctx0 = wrapper.commit_file('foo')
+    wrapper.command('cp', repo.root + b'/foo', repo.root + b'/bar')
+    wrapper.command('mv', repo.root + b'/foo', repo.root + b'/bar2')
+    ctx1 = wrapper.commit(rel_paths=['foo', 'bar', 'bar2'])
+
+    def fcp(find_renames=True, **opts):
+        return diff_fixture.find_changed_paths_commits(
+            [ctx1.hex()],
+            compare_to=[ctx0.hex()],
+            find_renames=find_renames,
+            **opts)
+
+    expected = {
+        b'bar': [(ChangedPaths.Status.COPIED, None)],
+        b'bar2': [(ChangedPaths.Status.RENAMED, None)],
+    }
+
+    assert fcp() == expected
+    assert fcp(diff_filters=[DS_ADDED]) == {}
+    assert fcp(diff_filters=[DS_RENAMED]) == sub_dict(expected, 'bar2')
+    assert fcp(diff_filters=[DS_COPIED]) == sub_dict(expected, 'bar')
 
 
 def test_get_patch_id(diff_fixture):
