@@ -24,6 +24,7 @@ from hgext3rd.heptapod.branch import (
     gitlab_branches,
 )
 
+from hgitaly.revision import ZERO_SHA_STR
 from hgitaly.stub.operations_pb2 import (
     OperationBranchUpdate,
     UserCommitFilesAction,
@@ -757,7 +758,7 @@ def test_user_commit_files(operations_fixture, tmpdir):
     assert exc_info.value.code() == StatusCode.ALREADY_EXISTS
 
     # invalid paths
-    for path in ('double//slash', '../../../good.joke', '/etc/shadow'):
+    for path in ('double//slash', '../../../good.joke'):
         with pytest.raises(RpcError) as exc_info:
             commit_files(
                 branch_name=gl_branch,
@@ -793,6 +794,21 @@ def test_user_commit_files(operations_fixture, tmpdir):
             )
         assert exc_info.value.code() == StatusCode.INVALID_ARGUMENT
 
+    # leading slash is interpreted as meaning the root of checkout
+    resp = commit_files(
+        branch_name=gl_branch,
+        commit_message=b'Deleted intopic',
+        actions=(dict(action=ActionType.CREATE,
+                      file_path=b'/etc/shadow',
+                      content=b"This is in repo!",
+                      ),
+                 )
+    )
+    hex11 = resp.branch_update.commit_id.encode()
+    wrapper.reload()
+    clone_pull_update(hex11)
+    assert (clone.path / 'etc/shadow').read_binary() == b"This is in repo!"
+
     # bogus request flows
     header_req = fixture.user_commit_files_header(
         branch_name=gl_other_branch,
@@ -817,3 +833,49 @@ def test_user_commit_files(operations_fixture, tmpdir):
             UserCommitFilesRequest(action=UserCommitFilesAction())
         ])
     assert exc_info.value.code() == StatusCode.INVALID_ARGUMENT
+
+
+def test_user_commit_files_null_old_oid(operations_fixture, tmpdir):
+    fixture = operations_fixture
+    commit_files = fixture.user_commit_files
+    ActionType = UserCommitFilesActionHeader.ActionType
+
+    fixture.hg_native = True  # TODO just remove hg-git now
+    fixture.heptapod_permission = 'publish'
+    wrapper = fixture.repo_wrapper
+    with open(wrapper.path / '.hg/hgrc', 'a') as hgrcf:
+        # this is part of the normally included `required.hgrc` of
+        # py-heptapod
+        hgrcf.write('\n'.join((
+            "[experimental]",
+            "topic.publish-bare-branch = yes"
+            ""
+        )))
+
+    gl_topic = b'topic/default/zetop'
+    gl_branch = b'branch/default'
+
+    wrapper.commit_file('foo')  # first commit on `branch/default`
+
+    # now let's start a topic, passing explicitely old_oid indicating
+    # that is does not exist yet
+    two_lines = b"zz\nsecond proto message"
+    resp = commit_files(
+        branch_name=gl_topic,
+        start_branch_name=gl_branch,
+        commit_message=b'New topic',
+        expected_old_oid=ZERO_SHA_STR,
+        actions=(dict(action=ActionType.CREATE,
+                      file_path=b'intop',
+                      content=two_lines,
+                      ),
+                 )
+    )
+    hex1 = resp.branch_update.commit_id.encode()
+
+    wrapper.reload()
+    ctx1 = wrapper.repo[hex1]
+
+    assert ctx1.description() == b'New topic'
+    assert ctx1.branch() == b'default'
+    assert ctx1.topic() == b'zetop'
