@@ -19,6 +19,14 @@ from mercurial_testhelpers import (
 )
 
 
+def delta_node(delta):  # pragma no cover (coverage_mercurial not ready for v7)
+    """Compatibility wrapper for changes in Mercurial 7.1"""
+    if isinstance(delta, tuple):  # hg<7.1
+        return delta[0]
+    else:  # hg>7.0
+        return delta.node
+
+
 def list_bundle_contents(path):
     """Simple utility to list the contents.
 
@@ -35,5 +43,6 @@ def list_bundle_contents(path):
                 version = part.params.get(b'version', b'01')
                 cg = changegroup.getunbundler(version, part, b'UN')
                 cg.changelogheader()
-                nodes.extend(node_hex(delta[0]) for delta in cg.deltaiter())
+                nodes.extend(node_hex(delta_node(delta))
+                             for delta in cg.deltaiter())
     return dict(nodes=nodes)
