@@ -43,6 +43,7 @@ from hgitaly.gitlab_ref import (
 )
 from hgitaly.servicer import SKIP_HOOKS_MD_KEY
 from hgitaly.stub.shared_pb2 import Repository
+from hgitaly.testing.grpc import wait_health_check
 from hgitaly.testing.storage import (
     DEFAULT_STORAGE_NAME,
     storage_path,
@@ -489,15 +490,21 @@ class RpcHelper(BaseRpcHelper):
         self.chunked_fields_remover = chunked_fields_remover
         self.chunks_concatenator = chunks_concatenator
 
-    def init_stubs(self):
-        comparison, stub_cls = self.comparison, self.stub_cls
+    def channel(self, vcs):
+        if vcs == 'git':
+            return self.comparison.gitaly_channel
 
         if self.hg_server == 'rhgitaly':
-            hg_channel = comparison.rhgitaly_channel
+            return self.comparison.rhgitaly_channel
         else:
-            hg_channel = comparison.hgitaly_channel
-        self.stubs = dict(git=stub_cls(comparison.gitaly_channel),
-                          hg=stub_cls(hg_channel))
+            return self.comparison.hgitaly_channel
+
+    def wait_health_check(self, vcs):
+        wait_health_check(self.channel(vcs))
+
+    def init_stubs(self):
+        self.stubs = {vcs: self.stub_cls(self.channel(vcs))
+                      for vcs in ('hg', 'git')}
 
     def hg2git(self, hg_sha):
         """Convert a Mercurial hex SHA to its counterpart SHA in Git repo.
