@@ -531,6 +531,8 @@ def test_compare_count_find_commits(gitaly_rhgitaly_comparison, hg_server):
         normalizer=normalizer,
         )
     rpc_helper.sorted = False
+    # if with RHGitaly, ensure that the sidecar is ready:
+    rpc_helper.wait_health_check('hg')
 
     @contextmanager
     def sorted_comparison():
@@ -581,6 +583,11 @@ def test_compare_count_find_commits(gitaly_rhgitaly_comparison, hg_server):
                 for r2 in all_revs:
                     assert_compare(revision=r1 + range_str + r2)
 
+    # with message_regex
+    rx = 'FOO.*zoO'  # tests case insensitivity
+    assert_compare(revision=ctx4.hex(), message_regex=rx)
+    assert_compare(all=True, message_regex=rx)
+
     # when `revision` is provided as a ref to a single commit
     refs = [b'', ctx0.hex(), b'topic/default/sampletop', ctx2.hex(),
             b'branch/stable', b'branch/default', b'HEAD']
