@@ -637,6 +637,11 @@ def test_compare_find_changed_paths(gitaly_comparison):
             ]
         )
 
+    DiffStatus = FindChangedPathsRequest.DiffStatus
+    DS_ADDED = DiffStatus.DIFF_STATUS_ADDED
+    DS_MODIFIED = DiffStatus.DIFF_STATUS_MODIFIED
+    DS_RENAMED = DiffStatus.DIFF_STATUS_RENAMED
+
     assert (do_rpc_tree('git', git_sha0, git_sha1, b'sub')
             ==
             do_rpc_tree('hg', ctx0.hex(), ctx1.hex(), b'sub')
@@ -644,10 +649,15 @@ def test_compare_find_changed_paths(gitaly_comparison):
 
     # case 1: actual test
     assert do_rpc_depr('git', [git_sha1]) == do_rpc_depr('hg', [ctx1.hex()])
-    assert (do_rpc_commits('git', [git_sha1])
-            ==
-            do_rpc_commits('hg', [ctx1.hex()])
-            )
+    for filters in ([],
+                    [DS_ADDED],
+                    [DS_MODIFIED],
+                    [DS_ADDED, DS_MODIFIED]
+                    ):
+        assert (do_rpc_commits('git', [git_sha1], diff_filters=filters)
+                ==
+                do_rpc_commits('hg', [ctx1.hex()], diff_filters=filters)
+                )
 
     # Note: khanchi97: As per the command used by gitaly for FindChangedPaths
     # cmd: `git diff-tree --stdin -z -m -r --name-status --no-renames`
@@ -768,10 +778,16 @@ def test_compare_find_changed_paths(gitaly_comparison):
                                   [(hg_sha5, b'zaz', 'blob')]))
 
     for find_renames in (False, True):
-        assert (do_rpc_commits('git', [git_sha5], find_renames=find_renames)
-                ==
-                do_rpc_commits('hg', [hg_sha5], find_renames=find_renames)
-                )
+        for filters in ([],
+                        [DS_MODIFIED],
+                        [DS_RENAMED],
+                        ):
+            kw = dict(find_renames=find_renames, diff_filters=filters)
+
+            assert (do_rpc_commits('git', [git_sha5], **kw)
+                    ==
+                    do_rpc_commits('hg', [hg_sha5], **kw)
+                    )
 
 
 def test_compare_get_patch_id(gitaly_comparison):
