@@ -44,6 +44,7 @@ from hgitaly.stub.shared_pb2 import (
 )
 from hgitaly.stub.commit_pb2_grpc import CommitServiceStub
 from hgitaly.stub.repository_pb2_grpc import RepositoryServiceStub
+from hgitaly.testing.grpc import wait_health_check
 from heptapod.testhelpers import (
     LocalRepoWrapper,
     git,
@@ -468,6 +469,7 @@ def test_compare_create_bundle_and_create_repository_from_bundle(
     # Actual test
     rpc_create_bundle('hg')
     rpc_create_bundle('git')
+    wait_health_check(hgitaly_channel)
     rpc_create_repository_from_bundle('hg')
     rpc_create_repository_from_bundle('git')
 
